@@ -5,11 +5,13 @@ both import directly from `bases`.
 
 """
 from datetime import datetime, date
-from typing import ClassVar, Type, Union, Optional, Dict
+from typing import Type, Optional, Dict, Union
 
 from .abstractions import W, AbstractJSONWizard
+from .bases import BaseMeta, M
 from .class_helper import (
-    _META_INITIALIZER, get_outer_class_name, get_class_name, create_new_class,
+    _META_INITIALIZER, _META,
+    get_outer_class_name, get_class_name, create_new_class,
     json_field_to_dataclass_field, dataclass_field_to_json_field
 )
 from .decorators import try_with_load
@@ -18,47 +20,29 @@ from .enums import LetterCase, DateTimeTo
 from .errors import ParseError
 from .loaders import get_loader
 from .log import LOG
-from .type_def import E
+from .type_def import E, T
 from .utils.type_conv import date_to_timestamp, as_enum
 
 
-class BaseJSONWizardMeta:
+# Internal mappings which help us keep a track of whether the `Meta`
+# config is already setup for a dataclass.
+#
+# Note: These are *only* for use with the ``fromdict`` and ``asdict``
+# helper functions.
+_LOAD_META: Dict[Type, Type[M]] = {}
+_DUMP_META: Dict[Type, Type[M]] = {}
+
+
+class BaseJSONWizardMeta(BaseMeta):
+    """
+    Superclass definition for the `JSONWizard.Meta` inner class.
+
+    See the implementation of the :class:`BaseMeta` class for the
+    available config that can be set.
+    """
+
     __slots__ = ()
 
-    # True to enable Debug mode for additional debug log output and more
-    # helpful messages during error handling.
-    debug_enabled: ClassVar[bool] = False
-
-    # A customized mapping of JSON keys to dataclass fields, that is used
-    # whenever `from_dict` or `from_json` is called.
-    #
-    # Note: this is in addition to the implicit field transformations, like
-    #   "myStr" -> "my_str"
-    #
-    # If the reverse mapping is also desired (i.e. dataclass field to JSON
-    # key), then specify the "__all__" key as a truthy value. If multiple JSON
-    # keys are specified for a dataclass field, only the first one provided is
-    # used in this case.
-    json_key_to_field: ClassVar[Dict[str, str]] = None
-
-    # How should :class:`time` and :class:`datetime` objects be serialized
-    # when converted to a Python dictionary object or a JSON string.
-    marshal_date_time_as: ClassVar[Union[DateTimeTo, str]] = None
-
-    # How JSON keys should be transformed to dataclass fields.
-    #
-    # Note that this only applies to keys which are to be set on dataclass
-    # fields; other fields such as the ones for `TypedDict` or `NamedTuple`
-    # sub-classes won't be similarly transformed.
-    key_transform_with_load: ClassVar[Union[LetterCase, str]] = None
-
-    # How dataclass fields should be transformed to JSON keys.
-    #
-    # Note that this only applies to dataclass fields; other fields such as
-    # the ones for `TypedDict` or `NamedTuple` sub-classes won't be similarly
-    # transformed.
-    key_transform_with_dump: ClassVar[Union[LetterCase, str]] = None
-
     @classmethod
     def _init_subclass(cls):
         """
@@ -85,6 +69,12 @@ class BaseJSONWizardMeta:
                         'these are global settings that will apply to all '
                         'JSONSerializable sub-classes.', get_class_name(cls))
 
+            # Copy over global defaults to the :class:`BaseMeta`
+            exclude_attrs = ('debug_enabled', )
+            for attr in BaseMeta.__annotations__:
+                if attr not in exclude_attrs:
+                    setattr(BaseMeta, attr, getattr(cls, attr, None))
+
             # Create a new class of `Type[W]`, and then pass `create=False` so
             # that we don't create new loader / dumper for the class.
             new_cls = create_new_class(cls, (AbstractJSONWizard, ))
@@ -159,6 +149,11 @@ class BaseJSONWizardMeta:
             cls_dumper.transform_dataclass_field = cls._as_enum_safe(
                 'key_transform_with_dump', LetterCase)
 
+        # Finally, save the meta config for the outer class. This will allow
+        # us to access this config as part of the JSON load/dump process if
+        # needed.
+        _META[outer_cls] = cls
+
     @classmethod
     def _as_enum_safe(cls, name: str, base_type: Type[E]) -> Optional[E]:
         """
@@ -177,3 +172,105 @@ class BaseJSONWizardMeta:
             e.class_name = get_class_name(cls)
             e.field_name = name
             raise
+
+
+# noinspection PyPep8Naming
+def LoadMeta(data_class: Type[T],
+             debug_enabled: bool = False,
+             raise_on_unknown_json_key: bool = False,
+             json_key_to_field: Dict[str, str] = None,
+             key_transform: Union[LetterCase, str] = None) -> M:
+    """
+    Helper function to setup the ``Meta`` Config for the JSON load
+    (de-serialization) process, which is intended for use alongside the
+    ``fromdict`` helper function.
+
+    For descriptions on what each of these params does, refer to the `Docs`_
+    below, or check out the :class:`BaseMeta` definition (I want to avoid
+    duplicating the descriptions for params here).
+
+    Examples::
+
+        >>> fromdict(MyClass, {"myStr": "value"}, \
+                     LoadMeta(MyClass, key_transform='CAMEL'))
+
+    .. _Docs: https://dataclass-wizard.readthedocs.io/en/latest/common_use_cases/meta.html
+    """
+    try:
+        return _LOAD_META[data_class]
+
+    except KeyError:
+        # Initial run - we have not set up the load config for the class yet
+
+        try:
+            cls_meta = _META[data_class]
+        except KeyError:
+            # Meta does not exist for the class. Create a new subclass of
+            # :class:`BaseMeta`, and cache it so we have it for next time
+            # (for example for the dump or serialization process).
+            cls_meta = create_new_class(
+                data_class, (BaseJSONWizardMeta, ), suffix='Meta')
+
+        # Set meta attributes here.
+        cls_meta.debug_enabled |= debug_enabled
+        cls_meta.raise_on_unknown_json_key = raise_on_unknown_json_key
+        cls_meta.json_key_to_field = json_key_to_field
+        cls_meta.key_transform_with_load = key_transform
+
+        # Run the meta initialization for the dataclass, and also save the
+        # mapping to `_META` so we have it for next time.
+        # noinspection PyProtectedMember
+        cls_meta._meta_initialize(data_class)
+
+        # Save the load config, so we don't have to run this logic each time.
+        _LOAD_META[data_class] = cls_meta
+
+
+# noinspection PyPep8Naming
+def DumpMeta(data_class: Type[T],
+             debug_enabled: bool = False,
+             marshal_date_time_as: Union[DateTimeTo, str] = None,
+             key_transform: Union[LetterCase, str] = None) -> M:
+    """
+    Helper function to setup the ``Meta`` Config for the JSON dump
+    (serialization) process, which is intended for use alongside the
+    ``asdict`` helper function.
+
+    For descriptions on what each of these params does, refer to the `Docs`_
+    below, or check out the :class:`BaseMeta` definition (I want to avoid
+    duplicating the descriptions for params here).
+
+    Examples::
+
+        >>> asdict(MyClass, {"myStr": "value"}, \
+                   DumpMeta(MyClass, key_transform='CAMEL'))
+
+    .. _Docs: https://dataclass-wizard.readthedocs.io/en/latest/common_use_cases/meta.html
+    """
+    try:
+        return _DUMP_META[data_class]
+
+    except KeyError:
+        # Initial run - we have not set up the dump config for the class yet
+
+        try:
+            cls_meta = _META[data_class]
+        except KeyError:
+            # Meta does not exist for the class. Create a new subclass of
+            # :class:`BaseMeta`, and cache it so we have it for next time
+            # (for example for the dump or serialization process).
+            cls_meta = create_new_class(
+                data_class, (BaseJSONWizardMeta, ), suffix='Meta')
+
+        # Set meta attributes here.
+        cls_meta.debug_enabled |= debug_enabled
+        cls_meta.marshal_date_time_as = marshal_date_time_as
+        cls_meta.key_transform_with_dump = key_transform
+
+        # Run the meta initialization for the dataclass, and also save the
+        # mapping to `_META` so we have it for next time.
+        # noinspection PyProtectedMember
+        cls_meta._meta_initialize(data_class)
+
+        # Save the dump config, so we don't have to run this logic each time.
+        _DUMP_META[data_class] = cls_meta
