@@ -27,3 +27,11 @@ _LOAD_HOOKS = '__LOAD_HOOKS__'
 # The name of the dictionary object that contains `dump` hooks for each
 # object type. Also used to check if a class is a :class:`BaseDumpHook`
 _DUMP_HOOKS = '__DUMP_HOOKS__'
+
+# Attribute name that will be defined for single-arg alias functions and
+# methods; mainly for internal use.
+SINGLE_ARG_ALIAS = '__SINGLE_ARG_ALIAS__'
+
+# Attribute name that will be defined for identity functions and methods;
+# mainly for internal use.
+IDENTITY = '__IDENTITY__'
