@@ -1,26 +1,27 @@
-__all__ = ['Parser',
+__all__ = ['IdentityParser',
+           'SingleArgParser',
+           'Parser',
            'LiteralParser',
            'UnionParser',
            'OptionalParser',
-           'ForwardRefParser',
            'IterableParser',
            'TupleParser',
            'VariadicTupleParser',
            'NamedTupleParser',
+           'NamedTupleUntypedParser',
            'MappingParser',
            'DefaultDictParser',
            'TypedDictParser']
 
-import sys
 from dataclasses import dataclass, InitVar
 from typing import (
-    _eval_type, Type, Any, Union, Optional, Tuple, Dict, Iterable, Callable
+    Type, Any, Optional, Tuple, Dict, Iterable, Callable, List
 )
 
 from .abstractions import AbstractParser, FieldToParser
 from .errors import ParseError
 from .type_def import (
-    FrozenKeys, NoneType, PyForwardRef, DefFactory,
+    FrozenKeys, NoneType, DefFactory,
     T, M, S, DD, LSQ, N, NT
 )
 from .utils.typing_compat import (
@@ -33,6 +34,28 @@ GetParserType = Callable[[Type[T], Type[T]], AbstractParser]
 TupleOfParsers = Tuple[AbstractParser, ...]
 
 
+@dataclass
+class IdentityParser(AbstractParser):
+    __slots__ = ()
+
+    def __call__(self, o: Any) -> T:
+        return o
+
+
+@dataclass
+class SingleArgParser(AbstractParser):
+    __slots__ = ('hook', )
+
+    hook: Callable[[Any], T]
+
+    def __post_init__(self, cls: Type[T]):
+        if not self.hook:
+            self.hook = lambda o: o
+
+    def __call__(self, o: Any) -> T:
+        return self.hook(o)
+
+
 @dataclass
 class Parser(AbstractParser):
     __slots__ = ('hook', )
@@ -40,18 +63,7 @@ class Parser(AbstractParser):
     hook: Callable[[Any, Type[T]], T]
 
     def __call__(self, o: Any) -> T:
-        try:
-            return self.hook(o, self.base_type)
-
-        except TypeError:
-            if self.hook is None:
-                err = TypeError('Provided type is not currently supported.')
-                raise ParseError(
-                    err, o, self.base_type,
-                    unsupported_type=self.base_type
-                )
-            # Else, raise the original error.
-            raise
+        return self.hook(o, self.base_type)
 
 
 @dataclass
@@ -154,26 +166,6 @@ class UnionParser(AbstractParser):
             o, [p.base_type for p in self.parsers])
 
 
-@dataclass
-class ForwardRefParser(AbstractParser):
-    __slots__ = ('parser', )
-
-    base_type: Union[str, 'PyForwardRef']
-    get_parser: InitVar[GetParserType]
-
-    def __post_init__(self, cls: Type[T], get_parser: GetParserType):
-        if isinstance(self.base_type, str):
-            self.base_type = PyForwardRef(self.base_type, is_argument=False)
-
-        # Evaluate the ForwardRef here
-        base_globals = sys.modules[cls.__module__].__dict__
-        self.base_type = _eval_type(self.base_type, base_globals, None)
-        self.parser = get_parser(self.base_type, cls)
-
-    def __call__(self, o: Any):
-        return self.parser(o)
-
-
 @dataclass
 class IterableParser(AbstractParser):
     """
@@ -257,6 +249,8 @@ class TupleParser(AbstractParser):
         #   that have `None` in the list of args.
         self.required_count: int = len(tuple(p for p in self.elem_parsers
                                              if None not in p))
+        if not self.elem_parsers:
+            self.elem_parsers = None
 
     def __call__(self, o: M) -> M:
         """
@@ -329,35 +323,61 @@ class VariadicTupleParser(TupleParser):
 @dataclass
 class NamedTupleParser(AbstractParser):
     __slots__ = ('hook',
-                 'field_to_parser')
+                 'field_to_parser',
+                 'field_parsers')
 
     base_type: Type[S]
-    hook: Callable[[Any, Type[NT], Optional[FieldToParser]], NT]
+    hook: Callable[
+        [Any, Type[NT], Optional[FieldToParser], List[AbstractParser]],
+        NT
+    ]
     get_parser: InitVar[GetParserType]
 
     def __post_init__(self, cls: Type[T], get_parser: GetParserType):
 
-        try:
-            # Get the field annotations for the `NamedTuple` type
-            type_anns: Dict[str, Type[T]] = get_named_tuple_field_types(
-                self.base_type
-            )
+        # Get the field annotations for the `NamedTuple` type
+        type_anns: Dict[str, Type[T]] = get_named_tuple_field_types(
+            self.base_type
+        )
 
-        except AttributeError:
-            self.field_to_parser: Optional[FieldToParser] = None
+        self.field_to_parser: Optional[FieldToParser] = {
+            f: get_parser(ftype, cls)
+            for f, ftype in type_anns.items()
+        }
 
-        else:
-            self.field_to_parser: Optional[FieldToParser] = {
-                f: get_parser(ftype, cls)
-                for f, ftype in type_anns.items()
-            }
+        self.field_parsers = list(self.field_to_parser.values())
 
     def __call__(self, o: Any):
         """
         Load a dictionary or list to a `NamedTuple` sub-class (or an
         un-annotated `namedtuple`)
         """
-        return self.hook(o, self.base_type, self.field_to_parser)
+        return self.hook(o, self.base_type,
+                         self.field_to_parser, self.field_parsers)
+
+
+@dataclass
+class NamedTupleUntypedParser(AbstractParser):
+    __slots__ = ('hook',
+                 'dict_parser',
+                 'list_parser')
+
+    base_type: Type[S]
+    hook: Callable[[Any, Type[NT], AbstractParser, AbstractParser], NT]
+    get_parser: InitVar[GetParserType]
+
+    def __post_init__(self, cls: Type[T], get_parser: GetParserType):
+
+        self.dict_parser = get_parser(dict, cls)
+        self.list_parser = get_parser(list, cls)
+
+    def __call__(self, o: Any):
+        """
+        Load a dictionary or list to a `NamedTuple` sub-class (or an
+        un-annotated `namedtuple`)
+        """
+        return self.hook(o, self.base_type,
+                         self.dict_parser, self.list_parser)
 
 
 @dataclass
