@@ -35,7 +35,7 @@ def as_bool(o: Union[str, bool, N]):
     return o.upper() in _TRUTHY_VALUES
 
 
-def as_int(o: Union[str, int, bool, None], base_type=int,
+def as_int(o: Union[str, int, float, bool, None], base_type=int,
            default=0, raise_=True):
     """
     Return `o` if already a int, otherwise return the int value for a
@@ -56,14 +56,24 @@ def as_int(o: Union[str, int, bool, None], base_type=int,
     if t is bool:
         raise TypeError(f'as_int: Incorrect type, object={o!r}, type={t}')
 
-    if not o:
-        return default
+    if t is float:
+        return base_type(round(o))
+
+    # Check if the string represents a float value, e.g. '2.7'
+    if t is str and '.' in o:
+        return base_type(round(float(o)))
 
     try:
         return base_type(o)
-    except ValueError:
+
+    except (TypeError, ValueError):
+
+        if not o:
+            return default
+
         if raise_:
             raise
+
         return default
 
 
@@ -84,9 +94,12 @@ def as_str(o: Union[str, None], base_type=str, raise_=True):
 
     try:
         return base_type(o)
+
     except ValueError:
+
         if raise_:
             raise
+
         return base_type()
 
 
