@@ -9,14 +9,14 @@ from dataclasses import dataclass
 from datetime import datetime, date, time
 from typing import (
     List, Optional, Union, Tuple, Dict, NamedTuple, Type, DefaultDict,
-    Set, FrozenSet
+    Set, FrozenSet, Generic
 )
 
 import pytest
 
-from dataclass_wizard import JSONSerializable, json_field, json_key
-from dataclass_wizard.errors import ParseError
-from dataclass_wizard.parsers import OptionalParser, Parser
+from dataclass_wizard import *
+from dataclass_wizard.errors import ParseError, MissingFields, UnknownJSONKey
+from dataclass_wizard.parsers import OptionalParser, Parser, IdentityParser, SingleArgParser
 from dataclass_wizard.type_def import NoneType, T
 from .conftest import MyUUIDSubclass
 from ..conftest import *
@@ -25,6 +25,96 @@ from ..conftest import *
 log = logging.getLogger(__name__)
 
 
+def test_fromdict():
+    """
+    Confirm that Meta settings for `fromdict` are applied as expected.
+    """
+
+    @dataclass
+    class MyClass:
+        my_bool: Optional[bool]
+        myStrOrInt: Union[str, int]
+
+    d = {'myBoolean': 'tRuE', 'my_str_or_int': 123}
+
+    c = fromdict(MyClass, d, LoadMeta(
+        MyClass,
+        key_transform='CAMEL',
+        json_key_to_field={'myBoolean': 'my_bool'}
+    ))
+
+    assert c.my_bool is True
+    assert isinstance(c.myStrOrInt, int)
+    assert c.myStrOrInt == 123
+
+
+def test_fromdict_raises_on_unknown_json_fields():
+    """
+    Confirm that Meta settings for `fromdict` are applied as expected.
+    """
+
+    @dataclass
+    class MyClass:
+        my_bool: Optional[bool]
+
+    d = {'myBoolean': 'tRuE', 'my_string': 'Hello world!'}
+    load_cfg = LoadMeta(MyClass,
+                        json_key_to_field={'myBoolean': 'my_bool'},
+                        raise_on_unknown_json_key=True)
+
+    # Technically we don't need to pass `load_cfg`, but we'll pass it in as
+    # that's how we'd typically expect to do it.
+    with pytest.raises(UnknownJSONKey) as exc_info:
+        _ = fromdict(MyClass, d, load_cfg)
+
+        e = exc_info.value
+
+        assert e.json_key == 'my_string'
+        assert e.obj == d
+        assert e.fields == ['my_bool']
+
+
+def test_fromdict_with_nested_dataclass():
+    """Confirm that `fromdict` works for nested dataclasses as well."""
+
+    @dataclass
+    class Container:
+        id: int
+        submittedDt: datetime
+        myElements: List['MyElement']
+
+    @dataclass
+    class MyElement:
+        order_index: Optional[int]
+        status_code: Union[int, str]
+
+    d = {'id': '123',
+         'submitted_dt': '2021-01-01 05:00:00',
+         'myElements': [
+             {'orderIndex': 111,
+              'statusCode': '200'},
+             {'order_index': '222',
+              'status_code': 404}
+         ]}
+
+    # Fix so the forward reference works (since the class definition is inside
+    # the test case)
+    globals().update(locals())
+
+    c = fromdict(Container, d, LoadMeta(Container,
+                                        key_transform='CAMEL'))
+
+    assert c.id == 123
+    assert c.submittedDt == datetime(2021, 1, 1, 5, 0)
+    # Key transform only applies to top-level dataclass
+    # unfortunately. Need to setup `LoadMeta` for `MyElement`
+    # if we need different key transform.
+    assert c.myElements == [
+            MyElement(order_index=111, status_code='200'),
+            MyElement(order_index=222, status_code=404)
+    ]
+
+
 @pytest.mark.parametrize(
     'input,expected',
     [
@@ -53,6 +143,28 @@ def test_bool(input, expected):
     assert result.my_bool == expected
 
 
+def test_from_dict_with_missing_fields():
+    """
+    Calling `from_dict` when required dataclass field(s) are missing in the
+    JSON object.
+    """
+
+    @dataclass
+    class MyClass(JSONSerializable):
+        my_str: str
+        MyBool: bool
+        my_int: int
+
+    value = 'Testing'
+    d = {'my_str': value, 'myBool': 'true'}
+
+    with pytest.raises(MissingFields) as e:
+        _ = MyClass.from_dict(d)
+
+    assert e.value.fields == ['my_str']
+    assert e.value.missing_fields == ['MyBool', 'my_int']
+
+
 def test_from_dict_key_transform_with_json_field():
     """
     Specifying a custom mapping of JSON key to dataclass field, via the
@@ -1051,14 +1163,36 @@ def test_optional_parser_contains(input, expected):
     assert actual == expected
 
 
+def test_single_arg_parser_without_hook():
+    """
+    Test case for `SingleArgParser` when the hook function is missing or None,
+    added for code coverage.
+
+    """
+    class MyClass(Generic[T]):
+        pass
+
+    parser = SingleArgParser(None, MyClass, None)
+
+    c = MyClass()
+    assert parser(c) == c
+
+
 def test_parser_with_unsupported_type():
     """
-    Test case for :meth:`Parser.__call__` with an unknown or unsupported
-    type, added for code coverage.
+    Test case for :meth:`LoadMixin.get_parser_for_annotation` with an unknown
+    or unsupported type, added for code coverage.
 
     """
-    base_type: Type[T] = str
-    mock_parser = Parser(None, base_type, hook=None)
+    class MyClass(Generic[T]):
+        pass
 
-    with pytest.raises(ParseError):
-        _ = mock_parser('hello world')
+    mock_parser = LoadMixin.get_parser_for_annotation(None, MyClass)
+
+    assert type(mock_parser) is IdentityParser
+
+    c = MyClass()
+    assert mock_parser(c) == c
+
+    # with pytest.raises(ParseError):
+    #     _ = mock_parser('hello world')
